﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace HelpForum
{
    public partial class DynamicTreeView : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if(!Page.IsPostBack)
            {
                BindData(); 
            }
        }

        private void BindData()
        {
            var categories = GetCategories();

            var treeNode = new TreeNode(); 

            BuildTree(categories, treeNode); 
          
            tvCategories.Nodes.Add(treeNode);
            tvCategories.DataBind(); 

        }

        private void BuildTree(IEnumerable list, TreeNode treeNode)
        {
            if (list == null) return;

            TreeNode tnAdd = null;

            foreach (var obj in list)
            {
                var nameProperty = obj.GetType().GetProperty("Name");
                if (nameProperty != null)
                {
                    var text = nameProperty.GetValue(obj, null).ToString();
                    tnAdd = new TreeNode() { Text = text };

                    var properties = obj.GetType().GetProperties(BindingFlags.Public | BindingFlags.Instance);
                    foreach (var property in properties)
                    {
                        var propValue = property.GetValue(obj, null);
                        BuildTree(propValue as IEnumerable, tnAdd);
                    }

                    if (tnAdd != null)
                        treeNode.ChildNodes.Add(tnAdd);

                }
            }

        }

        private List<Category> GetCategories()
        {
            var categories = new List<Category>(); 

            for(var i=1; i<=10; i++)
            {
                var category = new Category() {Name = "Category " + i}; 

                for(var j = 1; j<=10; j++)
                {
                    var product = new Product() {Name = "Product " + i}; 
                    category.Products.Add(product);
                }

                categories.Add(category);
            }

            return categories; 
        }
    }
}